<?php
/*
Plugin Name: SmoothGallery
Plugin URI: http://www.christianschenk.org/projects/wordpress-smoothgallery-plugin/
Description: Embeds JonDesign's SmoothGallery.
Version: 1.10.1
Author: Christian Schenk
Author URI: http://www.christianschenk.org/
*/

#
# WordPress SmoothGallery plugin
# Copyright (C) 2008 Christian Schenk
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#

# Identifier for various actions of this script (e.g. CSS)
define('SMOOTHGALLERY_ACTION', 'smoothgallery_action');
define('SMOOTHGALLERY_ACTION_CSS', 'css');
define('SMOOTHGALLERY_ACTION_IFRAME', 'iframe');
# The CSS file
define('SMOOTHGALLERY_CSS_FILE', dirname(__FILE__).'/css/jd.gallery.css');
# Path to this plugin
define('SMOOTHGALLERY_URL', '/wp-content/plugins/smoothgallery');
# Include the custom configuration
require_once('config.php');


/**
 * Parses the actions
 */
if (!empty($_REQUEST[SMOOTHGALLERY_ACTION])) {
	switch ($_REQUEST[SMOOTHGALLERY_ACTION]) {
		case SMOOTHGALLERY_ACTION_CSS:
			header('Content-type: text/css');
			$css = file_get_contents(SMOOTHGALLERY_CSS_FILE);
			$css = str_replace('<HEIGHT>', $_REQUEST['height'], $css);
			$css = str_replace('<WIDTH>', $_REQUEST['width'], $css);
			$css = str_replace('<BORDERCOLOR>', $_REQUEST['bordercolor'], $css);
			$css = str_replace('<URL>', $_REQUEST['prefix'], $css);
			echo $css;
			die();
			break;

		case SMOOTHGALLERY_ACTION_IFRAME:
			require_once('utils.php');
			# extract interesting parameters from the request
			$keys = array_merge(get_extra_smoothgallery_parameters(), get_smoothgallery_shortcode_atts());
			$atts = array();
			foreach ($keys as $key) $atts[$key] = $_REQUEST[$key];
			echo generate_iframe($atts, false);
			die();
			break;

		default:
			die();
			break;
	}
}


/**
 * Returns the parameters for the gallery if they're attached to a post or a
 * page, otherwise it will return default values.
 */
function get_smoothgallery_parameters() {
	# get default values
	require_once('utils.php');
	$defaults = get_default_smoothgallery_parameters();

	# overwrite default values with user-supplied data
	$globalRet = get_smoothgallery_global_parameters($defaults);
	$metaRet = get_smoothgallery_metadata($defaults);

	if ($globalRet === false and $metaRet === false) return null;

	return $defaults;
}


/**
 * If there're global parameters for the current page we'll use them.
 * If there're also parameters in a custom field they'll overwrite these defaults.
 *
 * @param array $defaults the defaults (passed by reference)
 * @return bool true if there're global parameters, otherwise false
 */
function get_smoothgallery_global_parameters(&$defaults) {
	$parameters = insertSmoothGallery();
	if ($parameters === false) return false;

	foreach ($parameters as $key => $value) {
		$defaults[$key] = $value;
	}

	return true;
}


/**
 * Examines the metadata for the current post or page and changes the default
 * values accordingly.
 *
 * @param array $defaults the defaults (passed by reference)
 * @return bool true if we changed the default values from $defaults, otherwise false
 */
function get_smoothgallery_metadata(&$defaults) {
	global $post;
	if (isset($post) == false) return false;

	# get the post's metadata and change the default values accordingly
	$meta = get_post_meta($post->ID, 'smoothgallery', true);
	if (empty($meta)) return false;

	$meta = strtolower($meta);
	if ($meta != '1' and $meta != 'on') {
		# the user may use these keys
		require_once('utils.php');
		$metaKeyMap = get_default_smoothgallery_parameters(false);
		foreach ($metaKeyMap as $key => $value) {
			$param = get_smoothgallery_parameter($meta, $value);
			if ($param !== false) $defaults[$key] = $param;
		}
	}

	return true;
}


/**
 * Adds a link to the CSS stylesheet in the header.
 */
function add_smoothgallery_css() {
	$parameters = get_smoothgallery_parameters();
	if (empty($parameters)) return;

	require_once('utils.php');
	echo get_smoothgallery_css($parameters);
}
if (function_exists('add_action')) add_action('wp_head', 'add_smoothgallery_css');


/**
 * This will add the JavaScript to the footer.
 */
function add_smoothgallery_js() {
	$parameters = get_smoothgallery_parameters();
	if (empty($parameters)) return;

	require_once('utils.php');
	echo get_smoothgallery_js($parameters);
}
if (function_exists('add_action')) add_action('wp_footer', 'add_smoothgallery_js');


/**
 * Adds a custom section to the "advanced" post/page edit screens that
 * contains the generated markup for the gallery.
 */
function smoothgallery_add_custom_box() {
	if(function_exists( 'add_meta_box' )) {
		add_meta_box( 'myplugin_sectionid', 'SmoothGallery', 'smoothgallery_inner_custom_box', 'post', 'advanced' );
		add_meta_box( 'myplugin_sectionid', 'SmoothGallery', 'smoothgallery_inner_custom_box', 'page', 'advanced' );
	} else {
		add_action('dbx_post_advanced', 'smoothgallery_old_custom_box' );
		add_action('dbx_page_advanced', 'smoothgallery_old_custom_box' );
	}
}
if (function_exists('add_action')) add_action('admin_menu', 'smoothgallery_add_custom_box');

/**
 * Prints the inner fields for the custom post/page section
 */
function smoothgallery_inner_custom_box() {
	require_once('utils.php');
	$images = get_images_for_post();

	# check whether there're images available
	$markup = $images; $rows = 1;
	if (is_array($images)) {
		$markup = generate_markup($images);
		$rows = 8;
	}
?>
	<label for="smoothgallery_code">If you want more control over the exact markup for the gallery, copy the following code into your content.<br/>Otherwise you may want to use the shortcode <tt>[smoothgallery]</tt>.</label><br/><br/>
	<textarea name="smoothgallery_code" cols="64" rows="<?php echo $rows; ?>"><?php echo $markup; ?></textarea>
<?php
}

/**
 * Prints the edit form for pre-WordPress 2.5 post/page
 */
function smoothgallery_old_custom_box() {
	echo '<div class="dbx-b-ox-wrapper">' . "\n";
	echo '<fieldset id="smoothgallery_fieldsetid" class="dbx-box">' . "\n";
	echo '<div class="dbx-h-andle-wrapper"><h3 class="dbx-handle">SmoothGallery</h3></div>';
	echo '<div class="dbx-c-ontent-wrapper"><div class="dbx-content">';
	// output editing form
	smoothgallery_inner_custom_box();
	echo "</div></div></fieldset></div>\n";
}


/**
 * Supplies the user with a shortcode that generates a gallery with the images
 * attached to the current post/page. If the parameter 'id' is given we'll
 * fetch the images from that post/page instead of the current one.
 */
function smoothgallery_shortcode($atts) {
	require_once('utils.php');
	$allAtts = array('id' => NULL, 'imgsize' => NULL, 'iframe' => NULL);
	foreach (get_extra_smoothgallery_parameters() as $key => $value) $allAtts[$value] = NULL;
	extract(shortcode_atts($allAtts, $atts));

	# sanity checks
	if (!is_numeric($id)) $id = NULL;
	if ($iframe !== NULL) $iframe = true;

	if ($iframe) {
		global $post;
		$atts['id'] = ($id == NULL) ? $post->ID : $id;
		$atts['imgsize'] = $imgsize;
		return generate_iframe($atts);
	}
	return generate_markup(get_images_for_post($id), $imgsize);
}
if (function_exists('add_shortcode')) add_shortcode('smoothgallery', 'smoothgallery_shortcode');

?>
