<?php

#
# WordPress SmoothGallery plugin
# Copyright (C) 2008 Christian Schenk
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#

/**
 * Returns an array with the parameters for the SmoothGallery.
 *
 * If the parameter $defaultValues is set to true the array contains just the
 * keys and the corresponding default values. If it's set to false it contains
 * the keys along with their meta-keys which may be used in the custom field.
 */
function get_default_smoothgallery_parameters($defaultValues = true) {
	# copied from jd.gallery.js
	# -> contains the default values and the keys that may be used along with the custom field
	$parameters = array('showArrows' => array('true', 'a', 'arrows', 'showArrows'),
	                    'showCarousel' => array('false', 'c', 'carousel', 'showCarousel'), # true by default
	                    'showInfopane' => array('true', 'i', 'info', 'infoPane', 'showInfopane'),
	                    'embedLinks' => array('true', 'l', 'links', 'embedLinks'),
	                    'fadeDuration' => array(500, 'fadeDuration'),
	                    'timed' => array('false', 't', 'timed'),
	                    'delay' => array(9000, 'd', 'delay'),
	                    'preloader' => array('true', 'preloader'),
	                    'preloaderImage' => array('true', 'preloaderImage'),
	                    'preloaderErrorImage' => array('true', 'preloaderErrorImage'),
	                    /* Data retrieval */
	                    'manualData' => array('[]', 'manualData'),
	                    'populateFrom' => array('false', 'populateFrom'),
	                    'populateData' => array('true', 'populateData'),
	                    'destroyAfterPopulate' => array('true', 'destroyAfterPopulate'),
	                    'elementSelector' => array('"div.imageElement"', 'elementSelector'),
	                    'titleSelector' => array('"h3"', 'titleSelector'),
	                    'subtitleSelector' => array('"p"', 'subtitleSelector'),
	                    'linkSelector' => array('"a.open"', 'linkSelector'),
	                    'imageSelector' => array('"img.full"', 'imageSelector'),
	                    'thumbnailSelector' => array('"img.thumbnail"', 'thumbnailSelector'),
	                    'defaultTransition' => array('"fade"', 'defaultTransition'),
	                    /* InfoPane options */
	                    'slideInfoZoneOpacity' => array(0.7, 'slideInfoZoneOpacity'),
	                    'slideInfoZoneSlide' => array('true', 'slideInfoZoneSlide'),
	                    /* Carousel options */
	                    'carouselMinimizedOpacity' => array(0.4, 'carouselMinimizedOpacity'),
	                    'carouselMinimizedHeight' => array(20, 'carouselMinimizedHeight'),
	                    'carouselMaximizedOpacity' => array(0.9, 'carouselMaximizedOpacity'),
	                    'thumbHeight' => array(75, 'thumbHeight'),
	                    'thumbWidth' => array(100, 'thumbWidth'),
	                    'thumbSpacing' => array(10, 'thumbSpacing'),
	                    'thumbIdleOpacity' => array(0.2, 'thumbIdleOpacity'),
	                    'textShowCarousel' => array("'Pictures'", 'textShowCarousel'),
	                    'showCarouselLabel' => array('true', 'showCarouselLabel'),
	                    'thumbCloseCarousel' => array('true', 'thumbCloseCarousel'),
	                    'useThumbGenerator' => array('false', 'useThumbGenerator'),
	                    'thumbGenerator' => array("'resizer.php'", 'thumbGenerator'),
	                    'useExternalCarousel' => array('false', 'useExternalCarousel'),
	                    'carouselElement' => array('false', 'carouselElement'),
	                    'carouselHorizontal' => array('true', 'carouselHorizontal'),
	                    'activateCarouselScroller' => array('true',' activateCarouselScroller'),
	                    'carouselPreloader' => array('true', 'carouselPreloader'),
	                    'textPreloadingCarousel' => array("'Loading...'", 'textPreloadingCarousel'),
	                    /* CSS Classes */
	                    'baseClass' => array('jdGallery', 'baseClass'),
	                    'withArrowsClass' => array('withArrows', 'withArrowsClass'),
	                    /* Plugins: HistoryManager */
	                    'useHistoryManager' => array('false', 'useHistoryManager'),
	                    'customHistoryKey' => array('false', 'customHistoryKey'));

	# extra parameters (e.g. for the CSS)
	$parameters['height'] = array(345, 'h', 'height');
	$parameters['width'] = array(460, 'w', 'width');
	$parameters['bordercolor'] = array('000', 'b', 'border', 'bordercolor');

	# return either the default values or the meta-keys
	$tmpParameters = array();
	foreach ($parameters as $key => $value) $tmpParameters[$key] = ($defaultValues == true) ? $value[0] : array_slice($value, 1);
	return $tmpParameters;
}


/**
 * Returns an array containing the names of some extra parameters that aren't a
 * part of SmoothGallery's core parameters.
 *
 * Hint: if you add extra parameters to get_default_smoothgallery_parameters(),
 * don't forget to add them here too.
 */
function get_extra_smoothgallery_parameters() {
	return array('height', 'width', 'bordercolor');
}


/**
 * Attributes that may be used along with the shortcode.
 */
function get_smoothgallery_shortcode_atts() {
	return array('id', 'prefix', 'imgsize');
}


/**
 * Returns the value from a string in the following format:
 * '<$param><delimiter><value>        [...]\n
 *  <$anotherparam><delimiter><value> [...]'
 * where delimiter will be either ':' or '='.
 *
 * @param string $data the string with the data
 * @param array $params the parameter we're searching for
 * @return mixed the value from the string, otherwise false
 */
function get_smoothgallery_parameter($data, $params) {
	$metas = explode("\n", $data);
	foreach ($metas as $meta) {
		foreach ($params as $param) {
			if (preg_match('/'.$param.' *[:=]{1} *(\S*)/i', $meta, $matches)) {
				if (empty($matches[1])) continue;
				return rtrim($matches[1]);
			}
		}
	}
	return false;
}


/**
 * Generates the SmoothGallery markup for an array of images.
 *
 * @returns false if the array is empty, otherwise the HTML markup
 */
function generate_markup($images, $imgsize = NULL) {
	if (empty($images) or ! is_array($images)) return false;

	$markup .= '<div id="myGallery">'."\n";
	foreach ($images as $image) {
		$imageSrc = $image->url;
		# search for another image size
		if ($imgsize != NULL) {
			foreach ($image->alt as $altImage) {
				if (strpos($altImage, $imgsize) === false) continue;
				$imageSrc = $altImage;
				break;
			}
		}

		$markup .= '<div class="imageElement">'."\n".
		           '<h3>'.$image->title.'</h3>'."\n".
		           '<p>'.$image->description.'</p>'."\n".
		           '<a href="'.$image->url.'" title="open image" class="open"></a>'."\n".
		           '<img src="'.$imageSrc.'" class="full" alt="'.$image->caption.'"/>'."\n".
		           #(($thumbnail != NULL) ? '<img src="'.$image->url.'" class="thumbnail" alt="'.$image->caption.'"/>'."\n" : '').
		           '</div>'."\n";
	}
	$markup .= '</div>';
	return $markup;
}


/**
 * Generates either the content for an iFrame or the tag itself that references
 * the iFrame.
 */
function generate_iframe($atts = NULL, $isTag = true) {
	# get the default values and overwrite them with the supplied attributes
	$defaults = get_default_smoothgallery_parameters();
	foreach ($atts as $key => $value) {
		$defaults[$key] = $value;
	}

	if ($isTag === true) {
		return '<iframe src="'.get_smoothgallery_action_url($defaults, SMOOTHGALLERY_ACTION_IFRAME).'&amp;id='.$atts['id'].(!empty($atts['imgsize']) ? '&amp;imgsize='.$atts['imgsize'] : '').
		       '" width="'.($defaults['width'] + ADD_PIXELS_TO_IFRAME).
		       '" height="'.($defaults['height'] + ADD_PIXELS_TO_IFRAME).
		       '" frameborder="0"></iframe>';
	}

	return "<html>\n<head>\n".get_smoothgallery_css($defaults)."\n</head>\n\n<body>\n".generate_markup(get_images_for_post($atts['id']), $atts['imgsize'])."\n\n".get_smoothgallery_js($defaults)."\n</body>\n</html>";
}


/**
 * Returns a link to our stylesheet.
 */
function get_smoothgallery_css($parameters = NULL) {
	return '<link rel="stylesheet" type="text/css" href="'.get_smoothgallery_action_url($parameters, SMOOTHGALLERY_ACTION_CSS).'" />';
}


/**
 * Returns the JavaScript.
 */
function get_smoothgallery_js($parameters) {
	$url = get_smoothgallery_url($parameters);
	$namespaced = ((SMOOTHGALLERY_USE_NAMESPACED) ? '.namespaced' : '');

	$result = '<script src="'.$url.'/scripts/mootools'.$namespaced.'.js" type="text/javascript"></script>'."\n".
	          '<script src="'.$url.'/scripts/jd.gallery'.$namespaced.'.js" type="text/javascript"></script>'."\n".
	          '<script type="text/javascript">'."\n".
	          'function startGallery() {'."\n".
	          "\tvar myGallery = new gallery($('myGallery'), {\n";

	$defaults = get_default_smoothgallery_parameters();
	$jsparams = "\t\t";
	foreach ($parameters as $key => $value) {
		# skip these extra keys
		if (in_array($key, get_extra_smoothgallery_parameters())) continue;
		if (in_array($key, get_smoothgallery_shortcode_atts())) continue;
		# deactivates the carousel by default (little hack)
		if ($key == 'showCarousel' and $value == 'false') $jsparams .= "$key: $value, ";
		# include a key and its value if it's different from the defaults
		if ($defaults[$key] != $value) $jsparams .= "$key: $value, ";
	}
	$result .= rtrim($jsparams, ' ,')."\n".
	           "\t})\n".
	           "}\n".
	           "window.addEvent('domready', startGallery);\n".
	           '</script>';

	return $result;
}


/**
 * Returns a URL that calls the smoothgallery.php script with some parameters.
 * Used to generate content like JavaScript, CSS or the content for an iFrame.
 */
function get_smoothgallery_action_url($parameters, $action) {
	if (empty($parameters) or is_array($parameters) == false) return '';
	if ($action != SMOOTHGALLERY_ACTION_CSS and $action != SMOOTHGALLERY_ACTION_IFRAME) return 'Unknown action!';
	$height = $parameters['height'];
	$width = $parameters['width'];
	$bordercolor = $parameters['bordercolor'];
	$url = get_smoothgallery_url($parameters);
	return $url.'/smoothgallery.php?'.SMOOTHGALLERY_ACTION.'='.$action.'&amp;prefix='.urlencode($url).'&amp;height='.$height.'&amp;width='.$width.'&amp;bordercolor='.$bordercolor;
}


/**
 * Returns the absolute URL to our plugin's folder. If this function gets
 * called outside "the loop" it tries to find the URL in the supplied array.
 */
function get_smoothgallery_url($parameters) {
	$wpUrl = (function_exists('get_bloginfo') ? get_bloginfo('wpurl') : $parameters['prefix']);
	if (strpos($wpUrl, SMOOTHGALLERY_URL) === false) return $wpUrl.SMOOTHGALLERY_URL;
	else return $wpUrl;
}


/**
 * Returns an array with pictures attached to the current post/page. You can
 * supply the ID of another post, so you don't have to search for the images
 * attached to the current post ($post->ID) only.
 */
function get_images_for_post($postId = NULL) {
	global $wpdb, $post;
	if (empty($wpdb)) include dirname(__FILE__).'/../../../wp-config.php'; # iFrame hack
	if (!is_numeric($postId)) $postId = NULL; # sanity check

	$sql = 'SELECT post_content AS description, post_title AS title, post_excerpt AS caption, guid AS url
	        FROM '.$wpdb->posts.'
	        WHERE post_parent = '.(($postId == NULL) ? $post->ID : $postId).'
	              AND post_type = "attachment"';
	$images = $wpdb->get_results($sql);
	if (empty($images)) return "There aren't any images attached here.";

	# tries to find images with another size
	for ($i = 0; $i < count($images); $i++) {
		$altImages = get_all_sizes_for_image($images[$i]->url);
		if (empty($altImages)) continue;
		$images[$i]->alt = $altImages;
	}

	# Reorders the images, so the image given in the request is the first image
	# in the gallery. Hint: we could have used "myGallery.goTo(<nr of image>)"
	# in the JavaScript too. This way it's currently easier though, because we
	# don't know the number of the image.
	if (!empty($_REQUEST['sgfi'])) {
		$firstImage = $_REQUEST['sgfi'];
		$tmpImages = array(); $tmpImages[] = '';
		for ($i = 0; $i < count($images); $i++) {
			if ($images[$i]->title == $firstImage) $tmpImages[0] = $images[$i];
			else $tmpImages[] = $images[$i];
		}
		$images = $tmpImages;
	}

	return $images;
}


/**
 * Searches the uploads folder for images denoted by the given URL but with
 * other sizes. Let's say the URL points at the image 'test.jpg', then we'll
 * try to find images like 'test-150x150.jpg' and the like.
 */
function get_all_sizes_for_image($url) {
	# get upload directory
	$imgDir = dirname(__FILE__).'/../../../'.get_option('upload_path');

	# extract the image's name
	$imgNameWithExt = end(explode('/', $url));
	$imgName = substr($imgNameWithExt, 0, strrpos($imgNameWithExt, '.'));

	$images = array();
	$allImages = listFiles($imgDir);
	foreach ($allImages as $image) {
		# the filename must match the name of the image followed by a dash
		# which is typically followed by the size of the image, e.g. 150x150.
		$image = end(explode('/', $image));
		if (strpos($image, $imgName.'-') === false) continue;
		# insert the filename into the given URL
		$images[] = str_replace($imgNameWithExt, $image, $url);
	}

	return $images;
}


/**
 * Copied from here:
 * http://php.net/manual/function.opendir.php#83990
 */
function listFiles($from = '.') {
	if(! is_dir($from)) return false;

	$files = array();
	$dirs = array($from);
	while(NULL !== ($dir = array_pop($dirs))) {
		if($dh = opendir($dir)) {
			while(false !== ($file = readdir($dh))) {
				if($file == '.' || $file == '..') continue;

				$path = $dir . '/' . $file;
				if(is_dir($path)) {
					$dirs[] = $path;
				} else {
					$files[] = $path;
				}
			}
			closedir($dh);
		}
	}
	return $files;
}

?>
